# -*- coding: utf-8 -*-

# Copyright © 2012-2013 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import unicode_literals
import codecs
import os

try:
    import docutils.core
    import docutils.io
    has_docutils = True
except ImportError:
    has_docutils = False

from nikola.plugin_categories import PageCompiler
from nikola.utils import get_logger, makedirs, req_missing

LOGGER = get_logger('compile_rest')


class CompileRest(PageCompiler):
    """Compile reSt into HTML."""

    name = "rest"

    def compile_html(self, source, dest, is_two_file=True):
        """Compile reSt into HTML."""

        if not has_docutils:
            req_missing(['docutils'], 'build this site (compile REST')
        makedirs(os.path.dirname(dest))
        error_level = 100
        with codecs.open(dest, "w+", "utf8") as out_file:
            with codecs.open(source, "r", "utf8") as in_file:
                data = in_file.read()
                if not is_two_file:
                    data = data.split('\n\n', 1)[-1]
                output, error_level, deps = rst2html(
                    data, settings_overrides={
                        'initial_header_level': 2,
                        'record_dependencies': True,
                        'stylesheet_path': None,
                        'link_stylesheet': True,
                        'syntax_highlight': 'short',
                        'math_output': 'mathjax',
                    })
                out_file.write(output)
            deps_path = dest + '.dep'
            if deps.list:
                with codecs.open(deps_path, "wb+", "utf8") as deps_file:
                    deps_file.write('\n'.join(deps.list))
            else:
                if os.path.isfile(deps_path):
                    os.unlink(deps_path)
        if error_level == 2:
            LOGGER.warning('Docutils reports warnings on {0}'.format(source))
        if error_level < 3:
            return True
        else:
            return False

    def create_post(self, path, onefile=False, **kw):
        metadata = {}
        metadata.update(self.default_metadata)
        metadata.update(kw)
        makedirs(os.path.dirname(path))
        with codecs.open(path, "wb+", "utf8") as fd:
            if onefile:
                for k, v in metadata.items():
                    fd.write('.. {0}: {1}\n'.format(k, v))
            fd.write("\nWrite your post here.")

    def set_site(self, site):
        for plugin_info in site.plugin_manager.getPluginsOfCategory("RestExtension"):
            if (plugin_info.name in site.config['DISABLED_PLUGINS']
                or (plugin_info.name in site.EXTRA_PLUGINS and
                    plugin_info.name not in site.config['ENABLED_EXTRAS'])):
                site.plugin_manager.removePluginFromCategory(plugin_info, "RestExtension")
                continue

            site.plugin_manager.activatePluginByName(plugin_info.name)
            plugin_info.plugin_object.set_site(site)
            plugin_info.plugin_object.short_help = plugin_info.description

        return super(CompileRest, self).set_site(site)


def rst2html(source, source_path=None, source_class=docutils.io.StringInput,
             destination_path=None, reader=None, reader_name='standalone',
             parser=None, parser_name='restructuredtext', writer=None,
             writer_name='html', settings=None, settings_spec=None,
             settings_overrides=None, config_section=None,
             enable_exit_status=None):
    """
    Set up & run a `Publisher`, and return a dictionary of document parts.
    Dictionary keys are the names of parts, and values are Unicode strings;
    encoding is up to the client.  For programmatic use with string I/O.

    For encoded string input, be sure to set the 'input_encoding' setting to
    the desired encoding.  Set it to 'unicode' for unencoded Unicode string
    input.  Here's how::

        publish_parts(..., settings_overrides={'input_encoding': 'unicode'})

    Parameters: see `publish_programmatically`.
    """
    output, pub = docutils.core.publish_programmatically(
        source=source, source_path=source_path, source_class=source_class,
        destination_class=docutils.io.StringOutput,
        destination=None, destination_path=destination_path,
        reader=reader, reader_name=reader_name,
        parser=parser, parser_name=parser_name,
        writer=writer, writer_name=writer_name,
        settings=settings, settings_spec=settings_spec,
        settings_overrides=settings_overrides,
        config_section=config_section,
        enable_exit_status=enable_exit_status)

    return pub.writer.parts['docinfo'] + pub.writer.parts['fragment'], pub.document.reporter.max_level, pub.settings.record_dependencies



# -*- coding: utf-8 -*-

# Copyright © 2012-2013 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""Implementation of compile_html based on pandoc.

You will need, of course, to install pandoc

"""

import codecs
import os
import subprocess

from nikola.plugin_categories import PageCompiler
from nikola.utils import req_missing, makedirs


class CompilePandoc(PageCompiler):
    """Compile markups into HTML using pandoc."""

    name = "pandoc"

    def compile_html(self, source, dest, is_two_file=True):
        makedirs(os.path.dirname(dest))
        try:
            subprocess.check_call(('pandoc', '-o', dest, source))
        except OSError as e:
            if e.strreror == 'No such file or directory':
                req_missing(['pandoc'], 'build this site (compile with pandoc)', python=False)

    def create_post(self, path, onefile=False, **kw):
        metadata = {}
        metadata.update(self.default_metadata)
        metadata.update(kw)
        makedirs(os.path.dirname(path))
        with codecs.open(path, "wb+", "utf8") as fd:
            if onefile:
                fd.write('<!-- \n')
                for k, v in metadata.items():
                    fd.write('.. {0}: {1}\n'.format(k, v))
                fd.write('-->\n\n')
            fd.write("Write your post here.")



# -*- coding: utf-8 -*-

# Path hack as shown by Kenneth Reitz at http://kennethreitz.com/repository-structure-and-python.html

import os
import sys
sys.path.insert(0, os.path.abspath('..'))

import nikola  # NOQA



# -*- coding: utf-8 -*-

# Copyright © 2012-2013 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import print_function
import os
import re
import sys
try:
    from urllib import unquote
    from urlparse import urlparse
except ImportError:
    from urllib.parse import unquote, urlparse  # NOQA

import lxml.html

from nikola.plugin_categories import Command
from nikola.utils import get_logger

LOGGER = get_logger('check')


class CommandCheck(Command):
    """Check the generated site."""

    name = "check"

    doc_usage = "-l [--find-sources] | -f"
    doc_purpose = "check links and files in the generated site"
    cmd_options = [
        {
            'name': 'links',
            'short': 'l',
            'long': 'check-links',
            'type': bool,
            'default': False,
            'help': 'Check for dangling links',
        },
        {
            'name': 'files',
            'short': 'f',
            'long': 'check-files',
            'type': bool,
            'default': False,
            'help': 'Check for unknown files',
        },
        {
            'name': 'clean',
            'long': 'clean-files',
            'type': bool,
            'default': False,
            'help': 'Remove all unknown files, use with caution',
        },
        {
            'name': 'find_sources',
            'long': 'find-sources',
            'type': bool,
            'default': False,
            'help': 'List possible source files for files with broken links.',
        },
    ]

    def _execute(self, options, args):
        """Check the generated site."""
        if not options['links'] and not options['files'] and not options['clean']:
            print(self.help())
            return False
        if options['links']:
            failure = self.scan_links(options['find_sources'])
        if options['files']:
            failure = self.scan_files()
        if options['clean']:
            failure = self.clean_files()
        if failure:
            sys.exit(1)

    existing_targets = set([])

    def analyze(self, task, find_sources=False):
        rv = False
        self.whitelist = [re.compile(x) for x in self.site.config['LINK_CHECK_WHITELIST']]
        try:
            filename = task.split(":")[-1]
            d = lxml.html.fromstring(open(filename).read())
            for l in d.iterlinks():
                target = l[0].attrib[l[1]]
                if target == "#":
                    continue
                parsed = urlparse(target)
                if parsed.scheme or target.startswith('//'):
                    continue
                if parsed.fragment:
                    target = target.split('#')[0]
                target_filename = os.path.abspath(
                    os.path.join(os.path.dirname(filename), unquote(target)))
                if any(re.match(x, target_filename) for x in self.whitelist):
                    continue
                elif target_filename not in self.existing_targets:
                    if os.path.exists(target_filename):
                        self.existing_targets.add(target_filename)
                    else:
                        rv = True
                        LOGGER.warn("Broken link in {0}: ".format(filename), target)
                        if find_sources:
                            LOGGER.warn("Possible sources:")
                            LOGGER.warn(os.popen('nikola list --deps ' + task, 'r').read())
                            LOGGER.warn("===============================\n")
        except Exception as exc:
            LOGGER.error("Error with:", filename, exc)
        return rv

    def scan_links(self, find_sources=False):
        LOGGER.notice("Checking Links:")
        LOGGER.notice("===============")
        failure = False
        for task in os.popen('nikola list --all', 'r').readlines():
            task = task.strip()
            if task.split(':')[0] in (
                    'render_tags', 'render_archive',
                    'render_galleries', 'render_indexes',
                    'render_pages'
                    'render_site') and '.html' in task:
                if self.analyze(task, find_sources):
                    failure = True
        if not failure:
            LOGGER.notice("All links checked.")
        return failure

    def scan_files(self):
        failure = False
        LOGGER.notice("Checking Files:")
        LOGGER.notice("===============\n")
        only_on_output, only_on_input = self.real_scan_files()
        if only_on_output:
            only_on_output.sort()
            LOGGER.warn("Files from unknown origins:")
            for f in only_on_output:
                LOGGER.warn(f)
            failure = True
        if only_on_input:
            only_on_input.sort()
            LOGGER.warn("Files not generated:")
            for f in only_on_input:
                LOGGER.warn(f)
        if not failure:
            LOGGER.notice("All files checked.")
        return failure

    def clean_files(self):
        only_on_output, _ = self.real_scan_files()
        for f in only_on_output:
            os.unlink(f)
        return True

    def real_scan_files(self):
        task_fnames = set([])
        real_fnames = set([])
        output_folder = self.site.config['OUTPUT_FOLDER']
        # First check that all targets are generated in the right places
        for task in os.popen('nikola list --all', 'r').readlines():
            task = task.strip()
            if output_folder in task and ':' in task:
                fname = task.split(':', 1)[-1]
                task_fnames.add(fname)
        # And now check that there are no non-target files
        for root, dirs, files in os.walk(output_folder):
            for src_name in files:
                fname = os.path.join(root, src_name)
                real_fnames.add(fname)

        only_on_output = list(real_fnames - task_fnames)

        only_on_input = list(task_fnames - real_fnames)

        return (only_on_output, only_on_input)



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Również dostępny w",
    "Archive": "Archiwum",
    "Categories": "Kategorie",
    "LANGUAGE": "polski",
    "More posts about": "Więcej postów o",
    "Newer posts": "Nowsze posty",
    "Next post": "Następny post",
    "Older posts": "Starsze posty",
    "Original site": "Oryginalna strona",
    "Posted": "Opublikowany",
    "Posts about %s": "Posty o %s",
    "Posts for year %s": "Posty z roku %s",
    "Posts for {month} {year}": "Posty z {month} {year}",
    "Previous post": "Poprzedni post",
    "Read in English": "Czytaj po polsku",
    "Read more": "Czytaj więcej",
    "Source": "Źródło",
    "Tags and Categories": "Tagi i Kategorie",
    "Tags": "Tags",
    "old posts page %d": "stare posty, strona %d",
}



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "LANGUAGE": "Ελληνικά",
    "Posts for year %s": "Αναρτήσεις για τη χρονιά %s",
    "Archive": "Αρχείο",
    "Posts about %s": "Αναρτήσεις για %s",
    "Tags": "Ετικέτες",
    "Also available in": "Διαθέσιμο και στο",
    "More posts about": "Περισσότερες αναρτήσεις για",
    "Posted": "Αναρτήθηκε",
    "Original site": "Ιστοσελίδα αρχικής ανάρτησης",
    "Read in English": "Διαβάστε στα Ελληνικά",
    "Newer posts": "Νεότερες αναρτήσεις",
    "Older posts": "Παλαιότερες αναρτήσεις",
    "Previous post": "Προηγούμενη ανάρτηση",
    "Next post": "Επόμενη ανάρτηση",
    "old posts page %d": "σελίδα παλαιότερων αναρτήσεων %d",
    "Read more": "Read more",
    "Source": "Source",
}



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "همچنین قابل دسترس از",
    "Archive": "آرشیو",
    "LANGUAGE": "فارسی",
    "More posts about": "ارسال‌های بیشتر دربارهٔ",
    "Newer posts": "ارسال‌های جدید‌تر",
    "Next post": "ارسال بعدی",
    "Older posts": "پست‌های قدیمی‌تر",
    "Original site": "سایت اصلی",
    "Posted": "ارسال شده",
    "Posts about %s": "ارسال‌ها دربارهٔ %s",
    "Posts for year %s": "ارسال‌ها برای سال %s",
    "Posts for {month} {year}": "ارسال برای {month} {year}",
    "Previous post": "ارسال پیشین",
    "Read in English": "به فارسی بخوانید",
    "Read more": "بیشتر بخوانید",
    "Source": "منبع",
    "Tags": "برچسب‌ها",
    "old posts page %d": "صفحهٔ ارسال‌های قدیمی %d",
}



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Също достъпно в",
    "Archive": "Архив",
    "Categories": "Категории",
    "LANGUAGE": "Български",
    "More posts about": "Още публикации относно ",
    "Newer posts": "Нови публикации",
    "Next post": "Следваща публикация",
    "Older posts": "Стари публикации",
    "Original site": "Оригиналния сайт",
    "Posted": "Публиковано",
    "Posts about %s": "Публикации относно %s",
    "Posts for year %s": "Публикации за %s година",
    "Posts for {month} {year}": "Публикации за {month} {year}",
    "Previous post": "Предишна публикация",
    "Read in English": "Прочетете на български",
    "Read more": "Прочети още",
    "Source": "Source",
    "Tags and Categories": "Тагове и Категории",
    "Tags": "Тагове",
    "old posts page %d": "стари публикации страница %d",
}



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Şu dilde de mevcut",
    "Archive": "Arşiv",
    "LANGUAGE": "Türkçe",
    "More posts about": "İlgili diğer yazılar",
    "Newer posts": "Daha yeni yazılar",
    "Next post": "Sonraki yazı",
    "Older posts": "Daha yeni yazılar",
    "Original site": "Orjinal site",
    "Posted": "Yayın tarihi",
    "Posts about %s": "Yazılar hakkında %s",
    "Posts for year %s": "Yıla göre yazılar %s",
    "Posts for {month} {year}": "{month} {year} göre yazılar",
    "Previous post": "Önceki yazı",
    "Read in English": "İngilizce olarak oku",
    "Read more": "Devamını oku",
    "Source": "Kaynak",
    "Tags": "Etiketler",
    "old posts page %d": "Geçmiş yazılar sayfası %d",
}



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Также доступно в",
    "Archive": "Архив",
    "LANGUAGE": "Русский",
    "More posts about": "Больше записей о",
    "Newer posts": "Новые записи",
    "Next post": "Следующая запись",
    "Older posts": "Старые записи",
    "Original site": "Оригинальный сайт",
    "Posted": "Опубликовано",
    "Posts about %s": "Записи с тэгом %s:",
    "Posts for year %s": "Записи за %s год",
    "Posts for {month} {year}": "",
    "Previous post": "Предыдущая запись",
    "Read in English": "Прочесть по-русски",
    "Read more": "Продолжить чтение",
    "Source": "Source",
    "Tags": "Тэги",
    "old posts page %d": "страница со старыми записями %d",
}



# -*- coding: utf-8 -*-

# Copyright © 2012-2013 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""Implementation of compile_html based on txt2tags.

Txt2tags is not in PyPI, you can install it with

easy_install -f "http://txt2tags.org/txt2tags.py#egg=txt2tags-2.6" txt2tags

"""

import codecs
import os

try:
    from txt2tags import exec_command_line as txt2tags
except ImportError:
    txt2tags = None  # NOQA

from nikola.plugin_categories import PageCompiler
from nikola.utils import makedirs, req_missing


class CompileTextile(PageCompiler):
    """Compile txt2tags into HTML."""

    name = "txt2tags"

    def compile_html(self, source, dest, is_two_file=True):
        if txt2tags is None:
            req_missing(['txt2tags'], 'build this site (compile txt2tags)')
        makedirs(os.path.dirname(dest))
        cmd = ["-t", "html", "--no-headers", "--outfile", dest, source]
        txt2tags(cmd)

    def create_post(self, path, onefile=False, **kw):
        metadata = {}
        metadata.update(self.default_metadata)
        metadata.update(kw)
        makedirs(os.path.dirname(path))
        with codecs.open(path, "wb+", "utf8") as fd:
            if onefile:
                fd.write("\n'''\n<!--\n")
                for k, v in metadata.items():
                    fd.write('.. {0}: {1}\n'.format(k, v))
                fd.write("-->\n'''\n")
            fd.write("\nWrite your post here.")



# coding: utf8
# Author: Rodrigo Bistolfi
# Date: 03/2013


""" Base class for Nikola test cases """


__all__ = ["BaseTestCase", "cd"]

from contextlib import contextmanager
import os
import sys
import unittest

import logbook

# Make logbook shutup
from nikola.utils import LOGGER

LOGGER.handlers.append(logbook.TestHandler())


if sys.version_info < (2, 7):

    try:
        import unittest2
        _unittest2 = True
    except ImportError:
        _unittest2 = False

    if _unittest2:
        BaseTestCase = unittest2.TestCase

    else:

        class BaseTestCase(unittest.TestCase):
            """ Base class for providing 2.6 compatibility """

            def assertIs(self, first, second, msg=None):
                self.assertTrue(first is second)

            def assertIsNot(self, first, second, msg=None):
                self.assertTrue(first is not second)

            def assertIsNone(self, expr, msg=None):
                self.assertTrue(expr is None)

            def assertIsNotNone(self, expr, msg=None):
                self.assertTrue(expr is not None)

            def assertIn(self, first, second, msg=None):
                self.assertTrue(first in second)

            def assertNotIn(self, first, second, msg=None):
                self.assertTrue(first not in second)

            def assertIsInstance(self, obj, cls, msg=None):
                self.assertTrue(isinstance(obj, cls))

            def assertNotIsInstance(self, obj, cls, msg=None):
                self.assertFalse(isinstance(obj, cls))


else:
    BaseTestCase = unittest.TestCase


@contextmanager
def cd(path):
    old_dir = os.getcwd()
    os.chdir(path)
    yield
    os.chdir(old_dir)



# -*- coding: utf-8 -*-

# Copyright © 2012-2013 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from copy import copy
import codecs
import string
import nikola.post

from nikola.plugin_categories import Task
from nikola import utils, rc4

LOGGER = utils.get_logger('render_posts')


def wrap_encrypt(path, password):
    """Wrap a post with encryption."""
    with codecs.open(path, 'rb+', 'utf8') as inf:
        data = inf.read() + "<!--tail-->"
    data = CRYPT.substitute(data=rc4.rc4(password, data))
    with codecs.open(path, 'wb+', 'utf8') as outf:
        outf.write(data)


class RenderPosts(Task):
    """Build HTML fragments from metadata and text."""

    name = "render_posts"

    def gen_tasks(self):
        """Build HTML fragments from metadata and text."""
        self.site.scan_posts()
        kw = {
            "translations": self.site.config["TRANSLATIONS"],
            "timeline": self.site.timeline,
            "default_lang": self.site.config["DEFAULT_LANG"],
            "hide_untranslated_posts": self.site.config['HIDE_UNTRANSLATED_POSTS'],
        }

        nikola.post.READ_MORE_LINK = self.site.config['READ_MORE_LINK']
        yield self.group_task()

        for lang in kw["translations"]:
            deps_dict = copy(kw)
            deps_dict.pop('timeline')
            for post in kw['timeline']:
                source = post.source_path
                dest = post.base_path
                if not post.is_translation_available(lang) and kw["hide_untranslated_posts"]:
                    continue
                else:
                    source = post.translated_source_path(lang)
                    if lang != post.default_lang:
                        dest = dest + '.' + lang
                task = {
                    'basename': self.name,
                    'name': dest,
                    'file_dep': post.fragment_deps(lang),
                    'targets': [dest],
                    'actions': [(self.site.get_compiler(post.source_path).compile_html,
                                [source, dest, post.is_two_file])],
                    'clean': True,
                    'uptodate': [utils.config_changed(deps_dict)],
                }
                if post.publish_later:
                    LOGGER.notice('{0} is scheduled to be published in the future ({1})'.format(post.source_path, post.date))
                if post.meta('password'):
                    task['actions'].append((wrap_encrypt, (dest, post.meta('password'))))
                yield task


CRYPT = string.Template("""\
<script>
function rc4(key, str) {
    var s = [], j = 0, x, res = '';
    for (var i = 0; i < 256; i++) {
        s[i] = i;
    }
    for (i = 0; i < 256; i++) {
        j = (j + s[i] + key.charCodeAt(i % key.length)) % 256;
        x = s[i];
        s[i] = s[j];
        s[j] = x;
    }
    i = 0;
    j = 0;
    for (var y = 0; y < str.length; y++) {
        i = (i + 1) % 256;
        j = (j + s[i]) % 256;
        x = s[i];
        s[i] = s[j];
        s[j] = x;
        res += String.fromCharCode(str.charCodeAt(y) ^ s[(s[i] + s[j]) % 256]);
    }
    return res;
}
function decrypt() {
    key = $$("#key").val();
    crypt_div = $$("#encr")
    crypted = crypt_div.html();
    decrypted = rc4(key, window.atob(crypted));
    if (decrypted.substr(decrypted.length - 11) == "<!--tail-->"){
        crypt_div.html(decrypted);
        $$("#pwform").hide();
        crypt_div.show();
    } else { alert("Wrong password"); };
}
</script>

<div id="encr" style="display: none;">${data}</div>
<div id="pwform">
<form onsubmit="javascript:decrypt(); return false;" class="form-inline">
<fieldset>
<legend>This post is password-protected.</legend>
<input type="password" id="key" placeholder="Type password here">
<button type="submit" class="btn">Show Content</button>
</fieldset>
</form>
</div>""")



# -*- coding: utf-8 -*-

# Copyright © 2012-2013 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""Implementation of compile_html based on CreoleWiki."""

import codecs
import os

try:
    from creole import Parser
    from creole.html_emitter import HtmlEmitter
    creole = True
except ImportError:
    creole = None

from nikola.plugin_categories import PageCompiler
from nikola.utils import makedirs, req_missing


class CompileWiki(PageCompiler):
    """Compile CreoleWiki into HTML."""

    name = "wiki"

    def compile_html(self, source, dest, is_two_file=True):
        if creole is None:
            req_missing(['creole'], 'build this site (compile CreoleWiki)')
        makedirs(os.path.dirname(dest))
        with codecs.open(dest, "w+", "utf8") as out_file:
            with codecs.open(source, "r", "utf8") as in_file:
                data = in_file.read()
                document = Parser(data).parse()
            output = HtmlEmitter(document).emit()
            out_file.write(output)

    def create_post(self, path, onefile=False, **kw):
        metadata = {}
        metadata.update(self.default_metadata)
        metadata.update(kw)
        makedirs(os.path.dirname(path))
        if onefile:
            raise Exception('There are no comments in CreoleWiki markup, so '
                            'one-file format is not possible, use the -2 '
                            'option.')
        with codecs.open(path, "wb+", "utf8") as fd:
            fd.write("Write your post here.")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import codecs
import shutil
import tempfile
import unittest
from os import path

from nikola.plugins.compile.markdown import CompileMarkdown


class FakeSite(object):
    config = {
        "MARKDOWN_EXTENSIONS": ['fenced_code', 'codehilite']
    }


class CompileMarkdownTests(unittest.TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        self.input_path = path.join(self.tmp_dir, 'input.markdown')
        self.output_path = path.join(self.tmp_dir, 'output.html')

        self.compiler = CompileMarkdown()
        self.compiler.set_site(FakeSite())

    def compile(self, input_string):
        with codecs.open(self.input_path, "w+", "utf8") as input_file:
            input_file.write(input_string)

        self.compiler.compile_html(self.input_path, self.output_path)

        output_str = None
        with codecs.open(self.output_path, "r", "utf8") as output_path:
            output_str = output_path.read()

        return output_str

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_compile_html_empty(self):
        input_str = ''
        actual_output = self.compile(input_str)
        self.assertEquals(actual_output, '')

    def test_compile_html_heading_tags(self):
        input_str = '''\
# header 1
## header 2
### header 3
#### header 4
##### header 5
###### header 6
'''
        expected_output = '''\
<h2>header 1</h2>
<h3>header 2</h3>
<h4>header 3</h4>
<h5>header 4</h5>
<h6>header 5</h6>
<h7>header 6</h7>
'''

        actual_output = self.compile(input_str)
        self.assertEquals(actual_output.strip(), expected_output.strip())

    def test_compile_html_code_hilite(self):
        input_str = '''\
    #!python
    from this
'''
        expected_output = '''\
<table class="codehilitetable"><tr><td class="linenos">\
<div class="linenodiv"><pre>1</pre></div>\
</td><td class="code"><div class="code">\
<pre><span class="kn">from</span> <span class="nn">this</span>
</pre></div>
</td></tr></table>
'''

        actual_output = self.compile(input_str)
        self.assertEquals(actual_output.strip(), expected_output.strip())

    def test_compile_html_gist(self):
        input_str = '''\
Here's a gist file inline:
[:gist: 4747847 zen.py]

Cool, eh?
'''
        expected_output = '''\
<p>Here's a gist file inline:
<div class="gist">
<script src="https://gist.github.com/4747847.js?file=zen.py"></script>
<noscript>
<pre>import this</pre>
</noscript>
</div>
</p>
<p>Cool, eh?</p>
'''
        actual_output = self.compile(input_str)
        self.assertEquals(actual_output.strip(), expected_output.strip())

    def test_compile_html_gist_2(self):
        input_str = '''\
Here's a gist file inline, using reStructuredText syntax:
..gist:: 4747847 zen.py

Cool, eh?
'''
        expected_output = '''\
<p>Here's a gist file inline, using reStructuredText syntax:
<div class="gist">
<script src="https://gist.github.com/4747847.js?file=zen.py"></script>
<noscript>
<pre>import this</pre>
</noscript>
</div>
</p>
<p>Cool, eh?</p>
'''
        actual_output = self.compile(input_str)
        self.assertEquals(actual_output.strip(), expected_output.strip())


if __name__ == '__main__':
    unittest.main()



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "",
    "Archive": "",
    "Categories": "",
    "LANGUAGE": "",
    "More posts about": "",
    "Newer posts": "",
    "Next post": "",
    "Older posts": "",
    "Original site": "",
    "Posted": "",
    "Posts about %s": "",
    "Posts for year %s": "",
    "Posts for {month} {year}": "",
    "Previous post": "",
    "Read in English": "",
    "Read more": "",
    "Source": "",
    "Tags and Categories": "",
    "Tags": "",
    "old posts page %d": "",
}



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Auch verfügbar in",
    "Archive": "Archiv",
    "Categories": "Kategorien",
    "LANGUAGE": "Deutsch",
    "More posts about": "Weitere Einträge über",
    "Newer posts": "Neuere Einträge",
    "Next post": "Nächster Eintrag",
    "Older posts": "Ältere Einträge",
    "Original site": "Original-Seite",
    "Posted": "Veröffentlicht",
    "Posts about %s": "Einträge über %s",
    "Posts for year %s": "Einträge aus dem Jahr %s",
    "Posts for {month} {year}": "Einträge für {month} {year}",
    "Previous post": "Vorheriger Eintrag",
    "Read in English": "Auf Deutsch lesen",
    "Read more": "Weiterlesen",
    "Source": "Source",
    "Tags and Categories": "Tags und Kategorien",
    "Tags": "Tags",
    "old posts page %d": "Vorherige Einträge %d",
}



# -*- coding: utf-8 -*-

# Copyright © 2012-2013 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import unicode_literals, print_function

import codecs
from collections import defaultdict
import os
import re
import string
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin  # NOQA

import lxml.html
try:
    import pyphen
except ImportError:
    pyphen = None

from .utils import (to_datetime, slugify, bytes_str, Functionary, LocaleBorg, unicode_str)

__all__ = ['Post']

TEASER_REGEXP = re.compile('<!--\s*TEASER_END(:(.+))?\s*-->', re.IGNORECASE)
READ_MORE_LINK = '<p class="more"><a href="{link}">{read_more}…</a></p>'


class Post(object):

    """Represents a blog post or web page."""

    def __init__(
        self, source_path, cache_folder, destination, use_in_feeds,
        translations, default_lang, base_url, messages, template_name,
        file_metadata_regexp=None, strip_indexes=False, index_file='index.html',
        tzinfo=None, current_time=None, skip_untranslated=False, pretty_urls=False,
        hyphenate=False,
    ):
        """Initialize post.

        The source path is the .txt post file. From it we calculate
        the meta file, as well as any translations available, and
        the .html fragment file path.
        """
        self.translated_to = set([])
        self._prev_post = None
        self._next_post = None
        self.base_url = base_url
        self.is_draft = False
        self.is_retired = False
        self.is_mathjax = False
        self.strip_indexes = strip_indexes
        self.index_file = index_file
        self.pretty_urls = pretty_urls
        self.source_path = source_path  # posts/blah.txt
        self.post_name = os.path.splitext(source_path)[0]  # posts/blah
        # cache/posts/blah.html
        self.base_path = os.path.join(cache_folder, self.post_name + ".html")
        self.metadata_path = self.post_name + ".meta"  # posts/blah.meta
        self.folder = destination
        self.translations = translations
        self.default_lang = default_lang
        self.messages = messages
        self.skip_untranslated = skip_untranslated
        self._template_name = template_name
        self.is_two_file = True
        self.hyphenate = hyphenate
        self._reading_time = None

        default_metadata = get_meta(self, file_metadata_regexp)

        self.meta = Functionary(lambda: None, self.default_lang)
        self.meta[default_lang] = default_metadata

        # Load internationalized metadata
        for lang in translations:
            if lang != default_lang:
                if os.path.isfile(self.source_path + "." + lang):
                    self.translated_to.add(lang)

                meta = defaultdict(lambda: '')
                meta.update(default_metadata)
                meta.update(get_meta(self, file_metadata_regexp, lang))
                self.meta[lang] = meta
            elif os.path.isfile(self.source_path):
                self.translated_to.add(default_lang)

        if not self.is_translation_available(default_lang):
            # Special case! (Issue #373)
            # Fill default_metadata with stuff from the other languages
            for lang in sorted(self.translated_to):
                default_metadata.update(self.meta[lang])

        if 'title' not in default_metadata or 'slug' not in default_metadata \
                or 'date' not in default_metadata:
            raise OSError("You must set a title (found '{0}'), a slug (found "
                          "'{1}') and a date (found '{2}')! [in file "
                          "{3}]".format(default_metadata.get('title', None),
                                        default_metadata.get('slug', None),
                                        default_metadata.get('date', None),
                                        source_path))

        # If timezone is set, build localized datetime.
        self.date = to_datetime(self.meta[default_lang]['date'], tzinfo)

        self.publish_later = False if current_time is None else self.date >= current_time

        is_draft = False
        is_retired = False
        self._tags = {}
        for lang in self.translated_to:
            self._tags[lang] = [x.strip() for x in self.meta[lang]['tags'].split(',')]
            self._tags[lang] = [t for t in self._tags[lang] if t]
            if 'draft' in self._tags[lang]:
                is_draft = True
                self._tags[lang].remove('draft')
            if 'retired' in self._tags[lang]:
                is_retired = True
                self._tags[lang].remove('retired')
            if 'private' in self._tags[lang]:
                is_retired = True
                self._tags[lang].remove('private')

        # While draft comes from the tags, it's not really a tag
        self.is_draft = is_draft
        self.is_retired = is_retired
        self.use_in_feeds = use_in_feeds and not is_draft and not is_retired \
            and not self.publish_later

        # If mathjax is a tag, then enable mathjax rendering support
        self.is_mathjax = 'mathjax' in self.tags

    def _has_pretty_url(self, lang):
        if self.pretty_urls and \
                self.meta[lang].get('pretty_url', '') != 'False' and \
                self.meta[lang]['slug'] != 'index':
            return True
        else:
            return False

    @property
    def alltags(self):
        """This is ALL the tags for this post."""
        tags = []
        for l in self._tags:
            tags.extend(self._tags[l])
        return list(set(tags))

    @property
    def tags(self):
        lang = self.current_lang()
        if lang in self._tags:
            return self._tags[lang]
        elif self.default_lang in self._tags:
            return self._tags[self.default_lang]
        else:
            return []

    @property
    def prev_post(self):
        lang = self.current_lang()
        rv = self._prev_post
        while self.skip_untranslated:
            if rv is None:
                break
            if rv.is_translation_available(lang):
                break
            rv = rv._prev_post
        return rv

    @prev_post.setter  # NOQA
    def prev_post(self, v):
        self._prev_post = v

    @property
    def next_post(self):
        lang = self.current_lang()
        rv = self._next_post
        while self.skip_untranslated:
            if rv is None:
                break
            if rv.is_translation_available(lang):
                break
            rv = rv._next_post
        return rv

    @next_post.setter  # NOQA
    def next_post(self, v):
        self._next_post = v

    @property
    def template_name(self):
        return self.meta('template') or self._template_name

    def _add_old_metadata(self):
        # Compatibility for themes up to Nikola 5.4.1
        # TODO: remove before Nikola 6
        self.pagenames = {}
        self.titles = {}
        for lang in self.translations:
            self.pagenames[lang] = self.meta[lang]['slug']
            self.titles[lang] = self.meta[lang]['title']

    def formatted_date(self, date_format):
        """Return the formatted date, as unicode."""
        fmt_date = self.date.strftime(date_format)
        # Issue #383, this changes from py2 to py3
        if isinstance(fmt_date, bytes_str):
            fmt_date = fmt_date.decode('utf8')
        return fmt_date

    def current_lang(self):
        """Return the currently set locale, if it's one of the
        available translations, or default_lang."""
        lang = LocaleBorg().current_lang
        if lang:
            if lang in self.translations:
                return lang
            lang = lang.split('_')[0]
            if lang in self.translations:
                return lang
        # whatever
        return self.default_lang

    def title(self, lang=None):
        """Return localized title.

        If lang is not specified, it will use the currently set locale,
        because templates set it.
        """
        if lang is None:
            lang = self.current_lang()
        return self.meta[lang]['title']

    def description(self, lang=None):
        """Return localized description."""
        if lang is None:
            lang = self.current_lang()
        return self.meta[lang]['description']

    def deps(self, lang):
        """Return a list of dependencies to build this post's page."""
        deps = []
        if self.default_lang in self.translated_to:
            deps.append(self.base_path)
        if lang != self.default_lang:
            deps += [self.base_path + "." + lang]
        deps += self.fragment_deps(lang)
        return deps

    def fragment_deps(self, lang):
        """Return a list of dependencies to build this post's fragment."""
        deps = []
        if self.default_lang in self.translated_to:
            deps.append(self.source_path)
        if os.path.isfile(self.metadata_path):
            deps.append(self.metadata_path)
        dep_path = self.base_path + '.dep'
        if os.path.isfile(dep_path):
            with codecs.open(dep_path, 'rb+', 'utf8') as depf:
                deps.extend([l.strip() for l in depf.readlines()])
        if lang != self.default_lang:
            lang_deps = list(filter(os.path.exists, [x + "." + lang for x in
                                                     deps]))
            deps += lang_deps
        return deps

    def is_translation_available(self, lang):
        """Return true if the translation actually exists."""
        return lang in self.translated_to

    def translated_source_path(self, lang):
        """Return path to the translation's source file."""
        if lang in self.translated_to:
            if lang == self.default_lang:
                return self.source_path
            else:
                return '.'.join((self.source_path, lang))
        elif lang != self.default_lang:
            return self.source_path
        else:
            return '.'.join((self.source_path, sorted(self.translated_to)[0]))

    def _translated_file_path(self, lang):
        """Return path to the translation's file, or to the original."""
        if lang in self.translated_to:
            if lang == self.default_lang:
                return self.base_path
            else:
                return '.'.join((self.base_path, lang))
        elif lang != self.default_lang:
            return self.base_path
        else:
            return '.'.join((self.base_path, sorted(self.translated_to)[0]))

    def text(self, lang=None, teaser_only=False, strip_html=False, really_absolute=False):
        """Read the post file for that language and return its contents.

        teaser_only=True breaks at the teaser marker and returns only the teaser.
        strip_html=True removes HTML tags
        lang=None uses the currently set locale

        All links in the returned HTML will be relative.
        The HTML returned is a bare fragment, not a full document.
        """

        if lang is None:
            lang = self.current_lang()
        file_name = self._translated_file_path(lang)
        with codecs.open(file_name, "r", "utf8") as post_file:
            data = post_file.read().strip()
        try:
            document = lxml.html.fragment_fromstring(data, "body")
        except lxml.etree.ParserError as e:
            # if we don't catch this, it breaks later (Issue #374)
            if str(e) == "Document is empty":
                return ""
            # let other errors raise
            raise(e)
        base_url = self.permalink(lang=lang, absolute=really_absolute)
        document.make_links_absolute(base_url)

        if self.hyphenate:
            hyphenate(document, lang)

        data = lxml.html.tostring(document, encoding='unicode')
        # data here is a full HTML doc, including HTML and BODY tags
        # which is not ideal (Issue #464)
        try:
            body = document.body
            data = (body.text or '') + ''.join(
                [lxml.html.tostring(child, encoding='unicode')
                    for child in body.iterchildren()])
        except IndexError:  # No body there, it happens sometimes
            pass

        if teaser_only:
            teaser = TEASER_REGEXP.split(data)[0]
            if teaser != data:
                if TEASER_REGEXP.search(data).groups()[-1]:
                    teaser += '<p class="more"><a href="{0}">{1}</a></p>'.format(
                        self.permalink(lang, absolute=really_absolute),
                        TEASER_REGEXP.search(data).groups()[-1])
                else:
                    teaser += READ_MORE_LINK.format(
                        link=self.permalink(lang, absolute=really_absolute),
                        read_more=self.messages[lang]["Read more"])
                # This closes all open tags and sanitizes the broken HTML
                document = lxml.html.fromstring(teaser)
                data = lxml.html.tostring(document, encoding='unicode')

        if data and strip_html:
            content = lxml.html.fromstring(data)
            data = content.text_content().strip()  # No whitespace wanted.
        return data

    @property
    def reading_time(self):
        """Reading time based on length of text.
        """
        if self._reading_time is None:
            text = self.text(strip_html=True)
            words_per_minute = 180
            words = len(text.split())
            self._reading_time = int(round(words / words_per_minute)) or 1
        return self._reading_time

    def source_link(self, lang=None):
        """Return absolute link to the post's source."""
        return "/" + self.destination_path(
            lang=lang,
            extension=self.source_ext(),
            sep='/')

    def destination_path(self, lang=None, extension='.html', sep=os.sep):
        """Destination path for this post, relative to output/.

        If lang is not specified, it's the current language.
        Extension is used in the path if specified.
        """
        if lang is None:
            lang = self.current_lang()
        if self._has_pretty_url(lang):
            path = os.path.join(self.translations[lang],
                                self.folder, self.meta[lang]['slug'], 'index' + extension)
        else:
            path = os.path.join(self.translations[lang],
                                self.folder, self.meta[lang]['slug'] + extension)
        if sep != os.sep:
            path = path.replace(os.sep, sep)
        return path

    def permalink(self, lang=None, absolute=False, extension='.html'):
        if lang is None:
            lang = self.current_lang()

        pieces = self.translations[lang].split(os.sep)
        pieces += self.folder.split(os.sep)
        if self._has_pretty_url(lang):
            pieces += [self.meta[lang]['slug'], 'index' + extension]
        else:
            pieces += [self.meta[lang]['slug'] + extension]
        pieces = [_f for _f in pieces if _f and _f != '.']
        link = '/' + '/'.join(pieces)
        if absolute:
            link = urljoin(self.base_url, link)
        index_len = len(self.index_file)
        if self.strip_indexes and link[-(1 + index_len):] == '/' + self.index_file:
            return link[:-index_len]
        else:
            return link

    def source_ext(self):
        return os.path.splitext(self.source_path)[1]

# Code that fetches metadata from different places


def re_meta(line, match=None):
    """re.compile for meta"""
    if match:
        reStr = re.compile('^\.\. {0}: (.*)'.format(re.escape(match)))
    else:
        reStr = re.compile('^\.\. (.*?): (.*)')
    result = reStr.findall(line.strip())
    if match and result:
        return (match, result[0])
    elif not match and result:
        return (result[0][0], result[0][1].strip())
    else:
        return (None,)


def _get_metadata_from_filename_by_regex(filename, metadata_regexp):
    """
    Tries to ried the metadata from the filename based on the given re.
    This requires to use symbolic group names in the pattern.

    The part to read the metadata from the filename based on a regular
    expression is taken from Pelican - pelican/readers.py
    """
    match = re.match(metadata_regexp, filename)
    meta = {}

    if match:
        # .items() for py3k compat.
        for key, value in match.groupdict().items():
            meta[key.lower()] = value  # metadata must be lowercase

    return meta


def get_metadata_from_file(source_path, lang=None):
    """Extracts metadata from the file itself, by parsing contents."""
    try:
        if lang:
            source_path = "{0}.{1}".format(source_path, lang)
        with codecs.open(source_path, "r", "utf8") as meta_file:
            meta_data = [x.strip() for x in meta_file.readlines()]
        return _get_metadata_from_file(meta_data)
    except (UnicodeDecodeError, UnicodeEncodeError):
        raise ValueError('Error reading {0}: Nikola only supports UTF-8 files'.format(source_path))
    except Exception:  # The file may not exist, for multilingual sites
        return {}


def _get_metadata_from_file(meta_data):
    """Parse file contents and obtain metadata.

    >>> g = _get_metadata_from_file
    >>> list(g([]).values())
    []
    >>> str(g(["FooBar","======"])["title"])
    'FooBar'
    >>> str(g(["#FooBar"])["title"])
    'FooBar'
    >>> str(g([".. title: FooBar"])["title"])
    'FooBar'
    >>> 'title' in g(["","",".. title: FooBar"])
    False
    >>> 'title' in g(["",".. title: FooBar"])  # for #520
    True

    """
    meta = {}

    re_md_title = re.compile(r'^{0}([^{0}].*)'.format(re.escape('#')))
    # Assuming rst titles are going to be at least 4 chars long
    # otherwise this detects things like ''' wich breaks other markups.
    re_rst_title = re.compile(r'^([{0}]{{4,}})'.format(re.escape(
        string.punctuation)))

    for i, line in enumerate(meta_data):
        # txt2tags requires an empty line at the beginning
        # and since we are here because it's a 1-file post
        # let's be flexible on what we accept, so, skip empty
        # first lines.
        if not line and i > 0:
            break
        if 'title' not in meta:
            match = re_meta(line, 'title')
            if match[0]:
                meta['title'] = match[1]
        if 'title' not in meta:
            if re_rst_title.findall(line) and i > 0:
                meta['title'] = meta_data[i - 1].strip()
        if 'title' not in meta:
            if re_md_title.findall(line):
                meta['title'] = re_md_title.findall(line)[0]

        match = re_meta(line)
        if match[0]:
            meta[match[0]] = match[1]

    return meta


def get_metadata_from_meta_file(path, lang=None):
    """Takes a post path, and gets data from a matching .meta file."""
    meta_path = os.path.splitext(path)[0] + '.meta'
    if lang:
        meta_path += '.' + lang
    if os.path.isfile(meta_path):
        with codecs.open(meta_path, "r", "utf8") as meta_file:
            meta_data = meta_file.readlines()
        while len(meta_data) < 6:
            meta_data.append("")
        (title, slug, date, tags, link, description) = [
            x.strip() for x in meta_data][:6]

        meta = {}

        if title:
            meta['title'] = title
        if slug:
            meta['slug'] = slug
        if date:
            meta['date'] = date
        if tags:
            meta['tags'] = tags
        if link:
            meta['link'] = link
        if description:
            meta['description'] = description

        return meta

    elif lang:
        # Metadata file doesn't exist, but not default language,
        # So, if default language metadata exists, return that.
        # This makes the 2-file format detection more reliable (Issue #525)
        return get_metadata_from_meta_file(path, lang=None)
    else:
        return {}


def get_meta(post, file_metadata_regexp=None, lang=None):
    """Get post's meta from source.

    If ``file_metadata_regexp`` is given it will be tried to read
    metadata from the filename.
    If any metadata is then found inside the file the metadata from the
    file will override previous findings.
    """
    meta = defaultdict(lambda: '')

    meta.update(get_metadata_from_meta_file(post.metadata_path, lang))

    if meta:
        return meta
    post.is_two_file = False

    if file_metadata_regexp is not None:
        meta.update(_get_metadata_from_filename_by_regex(post.source_path,
                                                         file_metadata_regexp))

    meta.update(get_metadata_from_file(post.source_path, lang))

    if lang is None:
        # Only perform these checks for the default language

        if 'slug' not in meta:
            # If no slug is found in the metadata use the filename
            meta['slug'] = slugify(unicode_str(os.path.splitext(
                os.path.basename(post.source_path))[0]))

        if 'title' not in meta:
            # If no title is found, use the filename without extension
            meta['title'] = os.path.splitext(
                os.path.basename(post.source_path))[0]

    return meta


def hyphenate(dom, lang):
    if pyphen is not None:
        hyphenator = pyphen.Pyphen(lang=lang)
        for tag in ('p', 'li', 'span'):
            for node in dom.xpath("//%s[not(parent::pre)]" % tag):
                insert_hyphens(node, hyphenator)
    return dom


def insert_hyphens(node, hyphenator):
    textattrs = ('text', 'tail')
    if isinstance(node, lxml.etree._Entity):
        # HTML entities have no .text
        textattrs = ('tail',)
    for attr in textattrs:
        text = getattr(node, attr)
        if not text:
            continue
        new_data = ' '.join([hyphenator.inserted(w, hyphen='\u00AD')
                             for w in text.split(' ')])
        # Spaces are trimmed, we have to add them manually back
        if text[0].isspace():
            new_data = ' ' + new_data
        if text[-1].isspace():
            new_data += ' '
        setattr(node, attr, new_data)

    for child in node.iterchildren():
        insert_hyphens(child, hyphenator)



# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Disponible aussi en",
    "Archive": "Archives",
    "LANGUAGE": "Français",
    "More posts about": "Plus de billets sur",
    "Newer posts": "Billets récents",
    "Next post": "Billet suivant",
    "Older posts": "Anciens billets",
    "Original site": "Site d'origine",
    "Posted": "Publié",
    "Posts about %s": "Billets sur %s",
    "Posts for year %s": "Billets de l'année %s",
    "Posts for {month} {year}": "Billets de {month} {year}",
    "Previous post": "Billet précédent",
    "Read in English": "Lire en français",
    "Read more": "Lire la suite",
    "Source": "Source",
    "Tags": "Étiquettes",
    "old posts page %d": "anciens billets page %d",
}
